<?php

/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

abstract class WC_Gateway_XPay_Generic_Method extends \WC_Payment_Gateway
{

    protected $selectedCard;
    protected $recurring;

    protected function __construct($id, $recurring)
    {
        $this->id = $id;
        $this->recurring = $recurring;

        $this->supports = array(
            'tokenization',
            'products',
            'refunds',
        );

        $this->init_form_fields();
        $this->init_settings();

        if (function_exists("wcs_is_subscription") && array_key_exists("nexi_xpay_recurring_enabled", $this->settings)) {
            if (($this->settings["nexi_xpay_recurring_enabled"] == "yes") && $this->recurring) {
                array_push(
                    $this->supports,
                    'subscriptions',
                    'subscription_cancellation',
                    'subscription_suspension',
                    'subscription_reactivation',
                    'subscription_amount_changes',
                    'subscription_date_changes',
                    'subscription_payment_method_change',
                );
            }
        }
    }

    public function process_payment($order_id)
    {
        $order = new \WC_Order($order_id);

        if (!empty($_REQUEST['installments'])) {
            update_post_meta($order_id, "installments", sanitize_text_field($_REQUEST['installments']));
        }

        return array(
            'result' => 'success',
            'redirect' => $order->get_checkout_payment_url(true),
        );
    }

    public function payment_fields()
    {
        if (class_exists("\WC_Subscriptions_Cart") && \WC_Subscriptions_Cart::cart_contains_subscription()) {
            echo __('Attention, the order for which you are making payment contains recurring payments, payment data will be stored securely by Nexi.', 'woocommerce-gateway-nexi-xpay');
        } else {
            echo $this->description;
        }
    }

    public function exec_payment($order_id)
    {
        $order = new \WC_Order($order_id);

        $recurringPaymentRequired = class_exists("\WC_Subscriptions_Cart") && \WC_Subscriptions_Cart::cart_contains_subscription();

        $order_form = \Nexi\WC_Gateway_XPay_API::getInstance()->get_payment_form($order, $this->selectedCard, $recurringPaymentRequired);

        echo "<form ";
        echo " action=\"" . htmlentities($order_form["target_url"]) . "\" ";
        echo " id=\"nexi_xpay_receipt_form\" ";
        echo " method=\"post\" ";
        echo " enctype=\"application/www-x-form-urlencoded\" ";
        echo " >";
        foreach ($order_form["fields"] as $name => $value) {
            // echo "<label>" . htmlentities($name) . "</label>";
            echo "<input type=\"hidden\" name=\"" . htmlentities($name) . "\" value=\"" . htmlentities($value) . "\" />";
            // echo "<br>";
        }
        echo "<input type=\"submit\" />";
        echo "</form>";
    }

    /**
     * on subscription renewal a new order is crated and post meta from the original one are copied and saved with new order's id as post_id
     * 
     * @param type $amount_to_charge
     * @param type $order
     */
    public function scheduled_subscription_payment($amount_to_charge, $order)
    {
        $num_contratto = get_xpay_post_meta($order->get_id(), 'num_contratto');
        $scadenza_pan = get_xpay_post_meta($order->get_id(), 'scadenza_pan');
        $currency = $order->get_currency();

        $newCodTrans = \Nexi\WC_Gateway_XPay_API::getInstance()->recurring_payment($num_contratto, $scadenza_pan, $amount_to_charge, $currency, $order);

        //must be uptaed otherwise refferrs to the the first payment
        update_post_meta($order->get_id(), '_xpay_' . "codTrans", $newCodTrans);

        $order->payment_complete($newCodTrans);
    }

    /**
     * order state is changed to "Refunded" automatically when total amount is refunded
     * 
     * @param type $order_id
     * @param type $amount
     * @param type $reaseon
     * @return type
     */
    public function process_refund($order_id, $amount = null, $reaseon = "")
    {
        if ($amount > 0) {
            $order = new \WC_Order($order_id);
            $cod_trans = get_xpay_post_meta($order->get_id(), 'codTrans');
            $currency = $order->get_currency();

            $res = \Nexi\WC_Gateway_XPay_API::getInstance()->refund($cod_trans, $amount, $currency);

            if ($res) {
                $orderDetails = \Nexi\WC_Gateway_XPay_API::getInstance()->order_detail($cod_trans);

                if ($orderDetails["stato"] == 'Annullato') {
                    $order->update_status('cancelled');
                }
            }

            return $res;
        }

        return new \WP_Error("invalid_refund_amount", __('Invalid refund amount.', 'woocommerce-gateway-nexi-xpay'));
    }

    public function init_form_fields()
    {
        if (!extension_loaded('bcmath')) {
            \WC_Admin_Settings::add_error(__('Attention, bcmath PHP extension is not loaded therefore the payment methods may not be available.', 'woocommerce-gateway-nexi-xpay'));
        }
    }

    /**
     * Generate Label HTML.
     *
     * @param mixed $key
     * @param mixed $data
     * @return string
     */
    public function generate_simple_label_html($key, $data)
    {
        $field = $this->plugin_id . $this->id . '_' . $key;

        ob_start();
?>
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr($field); ?>"><?php echo wp_kses_post($data['title']); ?></label>
            </th>
            <td class="forminp">
                <fieldset>
                    <label for="<?php echo esc_attr($field); ?>"><?php echo wp_kses_post($data['label']); ?></label>

                    <?php echo $this->get_description_html($data); ?>
                </fieldset>
            </td>
        </tr>
<?php
        return ob_get_clean();
    }
}
